#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#define DIM_W 800
#define DIM_H 600
#define CENTER_X DIM_W/2
#define CENTER_Y DIM_H/2
# define M_PI 3.14159265358979323846  /* non tutti i math.h definiscono il pi greco */

int main(int argc, char **argv)
{
	SDL_Surface *screen;	
	Uint32 black;

	TTF_Font *font;
	int height, width;
	SDL_Color text_color;
	char *string, *default_str = "Scritta con movimento sinusoidale";
	int size;
	char letter[2];
	SDL_Surface **letter_surf;
	SDL_Rect *letter_rect;
	char *font_file;
	
	int i, dir=1;
	double angle = 0;
	int xpos, first_x;
	int ypos;
	
	/* Operiamo il parsing dei parametri dati da shell */
        if (argc == 1) {
                font_file = "Neuropol.ttf";
		size = 20;
		string = default_str;
	}
        else if (argc == 4) {
		font_file = argv[1];
		if ((sscanf(argv[2], "%d", &size)) != 1) { 
			 printf("Il secondo argomento dev'essere un intero\n");
			 exit(-1);
		}
		string = argv[3];
	}
        else {
                printf("La sintassi corretta : %s font dimensione stringa\n", argv[0]);
                exit(-1);
        }

	/* Allocazione dinamica delle strutture in base al numero di lettere */ 
	letter_surf = (SDL_Surface **)malloc(sizeof(SDL_Surface *) * strlen(string));		
	letter_rect = (SDL_Rect *)malloc(sizeof(SDL_Rect) * strlen(string));
	
	/* Inizializzazione del sottosistema video */
        if (SDL_Init(SDL_INIT_VIDEO) < 0) {
                printf("Errore init SDL: %s\n", SDL_GetError());
                exit (-1);
        }
                                                                                                      
        /* All'uscita del programma viene eseguito SDL_Quit per risistemare le cose */
        atexit(SDL_Quit);

	/* Apriamo una finestra per disegnarci dentro */
        if ((screen = SDL_SetVideoMode(DIM_W, DIM_H, 0, SDL_HWSURFACE|SDL_DOUBLEBUF)) == NULL) {
                printf("Errore apertura video: %s\n", SDL_GetError());
                exit (-1);
        }

	/* Inizializziamo la SDL_ttf */
        if(TTF_Init()) {
                printf("Errore TTF_Init: %s", TTF_GetError());
                exit (-1);
        }

	/* Apriamo il font */
	if((font = TTF_OpenFont(font_file, size)) == NULL) {
		printf("Errore apertura del font: %s", TTF_GetError());
		exit (-1);
	}

	/* Mappiamo il colore di sfondo e del testo */
	black = SDL_MapRGB(screen->format, 0x00, 0x00, 0x00);
	
	text_color.r = 0xf2;
	text_color.g = 0x5b;
	text_color.b = 0x09;

	/* Ci torner utile conoscere le dimensioni della superficie dell'intero testo */
	TTF_SizeText(font, string, &width, &height);

	/* Creazione delle superfici per le singole lettere */	
	for (i=0; i < strlen(string); i++) {
		letter[0] = string[i];
		letter[1] = '\0';
		letter_surf[i] = TTF_RenderText_Blended(font, letter, text_color);
		letter_rect[i].w = letter_surf[i]->w;
		letter_rect[i].h = letter_surf[i]->h;
	}

	/* Scroll dal basso del testo */
	ypos = screen->h;
	xpos = CENTER_X - width/2;	
	while (ypos > CENTER_Y) {
		for (i=0; i < strlen(string); i++) {
			letter_rect[i].x = xpos;
			letter_rect[i].y = ypos;
			xpos += letter_rect[i].w;
			SDL_BlitSurface(letter_surf[i], NULL, screen, &letter_rect[i]);
		}
		xpos = CENTER_X - (width/2);
		ypos -= 2;
		
		SDL_Flip(screen);
		SDL_Delay(20);
		
		for (i=0; i < strlen(string); i++)
			SDL_FillRect(screen, &letter_rect[i], black);
	}

	/* Sinus scroller */
	first_x = xpos;
	while (angle <= 360*8) {
		for (i=0; i < strlen(string); i++) {
			if (i==0)
				xpos = first_x;
			letter_rect[i].x = xpos;
			xpos += letter_rect[i].w;
			ypos = CENTER_Y + sin(M_PI/180 * (angle + i*15)) * height;
			letter_rect[i].y = ypos;
			SDL_BlitSurface(letter_surf[i], NULL, screen, &letter_rect[i]);
		}
		angle += 7;
		
		SDL_Flip(screen);
		SDL_Delay(20);
		
		/* Rimbalzo agli estremi laterali dello schermo */
		if (xpos > screen->w)
			dir = -1;
		if (first_x < 0)
			dir = 1;
		first_x += dir*3;
		
		for (i=0; i < strlen(string); i++)
			SDL_FillRect(screen, &letter_rect[i], black);
	}

	/* Liberiamo le superfici */
	for (i=0; i < strlen(string); i++)
		SDL_FreeSurface(letter_surf[i]);
	/* Liberiamo le strutture create dinamicamente */
	free(letter_surf);
	free(letter_rect);
	
	/* Chiudiamo font e librerie */
	TTF_CloseFont(font);
	TTF_Quit();
	SDL_Quit();
	
	return 0;
}	
